require_relative 'result'

class Benchmark

  MESSAGE = 'processing time:'

  attr_reader :type, :msg
  attr_reader :data, :min, :max, :middle, :result, :results

  alias message msg
  alias time middle

  def initialize( type = :sec, message = nil )
    case type
      when :m, :min, :minute, :minutes then :min
      when :ms, :milli, :milliseconds  then :ms
      when :h, :hour, :hours           then :h
      else :sec
    end
    @data = []
    @type = type
    @msg  = message || MESSAGE
  end

  def call( message = nil, &operation )
    @msg = message unless message.nil?
    start = Time.now
    yield operation
    ends  = Time.now
    @data << [ start, ends ]
  end

  alias measure call

  def finish
    @results = Result.new @type, @msg
    @data.each do |st, en|
      @results.add st, en
    end
    @results.get
    @min     =  @results.min
    @max     =  @results.max
    @middle  =  @results.middle
    @result  =  @results.output
  end

  alias end finish

  # TO-DO
  # save output
end
